#ifndef GENHUM_STRUCT_H
#define GENHUM_STRUCT_H


/* ------------------------------------------------------------
 * Generic structures
 */

typedef struct GENHUM_MATRIX {
  double mat[16]; /* m0     m1     m2     m3=tx
		     m4     m5     m6     m7=ty
		     m8     m9     m10    m11=tz
		     m12=0  m13=0  m14=0  m15=1 */
} GENHUM_MATRIX;


/* ------------------------------------------------------------
 * Limbs
 */

/* List of all limbs */
typedef enum GENHUM_LIMB_NAME {
  GENHUM_LIMB_TORSO, /* 0 */
  GENHUM_LIMB_HEAD,  /* 1 */

  GENHUM_LIMB_ARM_UPPER_LEFT,  /* 2 */
  GENHUM_LIMB_ARM_UPPER_RIGHT, /* 3 */

  GENHUM_LIMB_ARM_FORE_LEFT,  /* 4 */
  GENHUM_LIMB_ARM_FORE_RIGHT, /* 5 */

  GENHUM_LIMB_HAND_LEFT,  /* 6 */
  GENHUM_LIMB_HAND_RIGHT, /* 7 */

  GENHUM_LIMB_LEG_THIGH_LEFT,  /* 8 */
  GENHUM_LIMB_LEG_THIGH_RIGHT, /* 9 */

  GENHUM_LIMB_LEG_LOWER_LEFT,  /* 10 */
  GENHUM_LIMB_LEG_LOWER_RIGHT, /* 11 */

  GENHUM_LIMB_FOOT_LEFT,  /* 12 */
  GENHUM_LIMB_FOOT_RIGHT, /* 13 */

  GENHUM_NB_LIMBS /* 14 limbs */ /* !!! WARNING: must be an _even_ number !!! */

} GENHUM_LIMB_NAME;


/* Geometry of one limb */
typedef struct GENHUM_LIMB_GEOM {
  double topRa;
  double topRb;
  double downRa;
  double downRb;
  double length;
} GENHUM_LIMB_GEOM;

/* Geometry of every human limbs */
typedef struct GENHUM_HUMAN_LIMBS_GEOM {
  GENHUM_LIMB_GEOM limb[GENHUM_NB_LIMBS];
} GENHUM_HUMAN_LIMBS_GEOM;

typedef struct GENHUM_HUMAN_POSITION {
  double x;
  double y;
  double yaw;
} GENHUM_HUMAN_POSITION;

/* Human posture : configuration of every human limbs 
   - ABSOLUTE configs are expressed in TORSO frame
   - RELATIVE configs are expressed in PARENT-CHILD LINK frame 
   - For the TORSO it is the ROBOT frame
*/
typedef struct GENHUM_HUMAN_POSTURE {
  GENHUM_HUMAN_POSITION pos;
  int            confidence[GENHUM_NB_LIMBS]; /* value in [0..100] */
  //  POM_EULER      relativeEuler[GENHUM_NB_LIMBS];
  GENHUM_MATRIX  relativeMat[GENHUM_NB_LIMBS];
  GENHUM_MATRIX  absoluteMat[GENHUM_NB_LIMBS];
} GENHUM_HUMAN_POSTURE;


/* ------------------------------------------------------------
 * Links
 */

/* List of links */
typedef enum GENHUM_LINK_NAME {
  GENHUM_LINK_TORSO_HEAD,
  GENHUM_LINK_TORSO_ARM_UPPER_LEFT,
  GENHUM_LINK_TORSO_ARM_UPPER_RIGHT,
  GENHUM_LINK_TORSO_LEG_THIGH_LEFT,
  GENHUM_LINK_TORSO_LEG_THIGH_RIGHT,

  GENHUM_LINK_ARM_UPPER_LEFT_ARM_FORE_LEFT,
  GENHUM_LINK_ARM_UPPER_RIGHT_ARM_FORE_RIGHT,
  GENHUM_LINK_ARM_FORE_LEFT_HAND_LEFT,
  GENHUM_LINK_ARM_FORE_RIGHT_HAND_RIGHT,

  GENHUM_LINK_LEG_THIGH_LEFT_LEG_LOWER_LEFT,
  GENHUM_LINK_LEG_THIGH_RIGHT_LEG_LOWER_RIGHT,
  GENHUM_LINK_LEG_LOWER_LEFT_FOOT_LEFT,
  GENHUM_LINK_LEG_LOWER_RIGHT_FOOT_RIGHT,

  GENHUM_NB_LINKS
} GENHUM_LINK_NAME;


/* Position of a child link on the parent limb frame */
typedef struct GENHUM_CHILD_LINK_POSITION {
  GENHUM_LIMB_NAME fromParent;
  GENHUM_LIMB_NAME toChild;
  GENHUM_MATRIX    transf;
} GENHUM_CHILD_LINK_POSITION;

/* Mobility description */
typedef struct GENHUM_CHILD_LINK_MOBILITY {
  GENHUM_LIMB_NAME fromParent;
  GENHUM_LIMB_NAME toChild;
  double yawMin, pitchMin, rollMin;  /* Values in [-Pi; Pi] */
  double yawMax, pitchMax, rollMax;  /* Values in [-Pi; Pi] */
} GENHUM_CHILD_LINK_MOBILITY;


typedef struct GENHUM_HUMAN_LINKS {
  GENHUM_CHILD_LINK_POSITION position[GENHUM_NB_LINKS];
/*   GENHUM_CHILD_LINK_MOBILITY mobility[GENHUM_NB_LINKS]; - not used yet */
} GENHUM_HUMAN_LINKS;

typedef struct GENHUM_HUMAN_DISTANCE {
  double minDistRobToHum;
  double costDistRobToHum;
} GENHUM_HUMAN_DISTANCE;


#endif /* GENHUM_STRUCT_H */
